/* --------------------------- k_cext  ----------------------------------- */
/*                                                                              */
/* Program c directly within a pd object.
/*                                                                              */
/* This program is free software; you can redistribute it and/or                */
/* modify it under the terms of the GNU General Public License                  */
/* as published by the Free Software Foundation; either version 2               */
/* of the License, or (at your option) any later version.                       */
/*                                                                              */
/* This program is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of               */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                */
/* GNU General Public License for more details.                                 */
/*                                                                              */
/* You should have received a copy of the GNU General Public License            */
/* along with this program; if not, write to the Free Software                  */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  */
/*                                                                              */
/* ---------------------------------------------------------------------------- */


#include "m_pd.h"

#include "k_cext.h"

static char *version = "k_cext v0.1, written by Kjetil S. Matheussen, k.s.matheussen@notam02.no";


static void k_cext_float(t_k_cext *x, t_floatarg f)
{
  /*
  int i;
  for(i=0;i<5;i++){
    post("%d: %f\n",i,x->values[i]);
  }
  */
  x->values[0]=f;

  if(x->k_cext_process!=NULL)
    (*x->k_cext_process)(x);
}

static t_class *k_cext_class;

static void k_cext_free(t_k_cext *x)
{

  if(x->handle!=NULL)
    dlclose(x->handle);
}

static void *k_cext_new(t_symbol *s, t_int argc, t_atom* argv)
{
  int i;
  FILE *file;char temp[500];

  t_k_cext *x = (t_k_cext *)pd_new(k_cext_class);

  x->num_ins=atom_getfloatarg(0,argc,argv);
  x->num_outs=atom_getfloatarg(1,argc,argv);

  for(i=0;i<x->num_ins;i++){
    x->inlets[i] = floatinlet_new(&x->x_ob, &x->values[i+1]);
  }

  for(i=0;i<x->num_outs;i++){
    x->outlets[i] = outlet_new(&x->x_ob, gensym("float"));
  }

  x->name=calloc(1,50);
  sprintf(x->name,"/tmp/pd-XXXXXX");

  mktemp(x->name);

  x->name[strlen(x->name)+2]=0;
  x->name[strlen(x->name)+1]='c';
  x->name[strlen(x->name)]='.';


  file=fopen(x->name,"w");
  //  post("name: %s\n",x->name);

  fprintf(file,"#include \"" INCLUDEPATH "/" LINUXINCLUDE "/m_pd.h\"\n");
  fprintf(file,"#include \"" INCLUDEPATH  "/k_cext.h\"\n");
  fprintf(file,"#define V(a) (x->values[a])\n");
  fprintf(file,"#define OUT(a,b) outlet_float(x->outlets[a],b)\n");

  fprintf(file,"void k_cext_process(t_k_cext *x){\n");
  for(i=3;i<argc;i++){
    switch(argv[i].a_type){
    case 1:
      fprintf(file,"%ff\n",atom_getfloatarg(i,argc,argv));
      break;
    case 2:
      fprintf(file,"%s\n",atom_getsymbolarg(i,argc,argv)->s_name);
      break;
    default:
      break;
    }
  }
  fprintf(file,"}\n");

  fclose(file);

  sprintf(temp,"gcc -w %s -o %s.o -shared",x->name,x->name);
  system(temp);

  sprintf(x->name,"%s.o",x->name);

  x->handle=dlopen(x->name,RTLD_NOW|RTLD_GLOBAL);
  if(x->handle!=NULL){
    x->k_cext_process=dlsym(x->handle,"k_cext_process");
  }else{
    x->k_cext_process=NULL;
    post("Error.\n");
  }

  unlink(x->name);

  x->name[strlen(x->name)-2]=0;
  unlink(x->name);

  free(x->name);

  post(version);
  return (void *)x;
}

void k_cext_setup(void)
{
    k_cext_class = class_new(gensym("k_cext"), (t_newmethod)k_cext_new,
    	(t_method)k_cext_free, sizeof(t_k_cext), 0, A_GIMME, 0);

    class_addfloat(k_cext_class, k_cext_float);

    //    class_addmethod(k_cext_class, (t_method)k_cext_ft1, gensym("ft1"), A_FLOAT, 0);

    class_sethelpsymbol(k_cext_class, gensym("help-k_cext.pd"));
}

